#pragma once

#include "camera.h"
#include "glew.h"
#include "window.h"
#include "shader.h"
#include "vao.h"
#include "fbo.h"
#include "rbo.h"
#include "vbo.h"
#include "vertexattribute.h"
#include "debug.h"

#define RENDERER_BUFFER_COUNT RENDERER_BUFFER_UI + 1
typedef enum RendererBuffer
{
	RENDERER_BUFFER_WORLD,
	RENDERER_BUFFER_UI
} RendererBuffer;

typedef struct Renderer
{
	SDL_GLContext glContext;
	Window* window;
	Camera camera;
	Texture fboTextures[RENDERER_BUFFER_COUNT];
	FBO fbos[RENDERER_BUFFER_COUNT];
	RBO rbos[RENDERER_BUFFER_COUNT];
	VBO vbo;	
	VBO ebo;	
	VAO vao;	
	ivec2 size;
} Renderer;

void renderer_init(Renderer* self, Window* window, CameraType type, ivec2 size);
void renderer_clear_color_set(Renderer* self, vec4 color);
void renderer_clear(Renderer* self);
void renderer_shader_use(Renderer* self, Shader* shader);
void renderer_buffer_use(Renderer* self, RendererBuffer buffer);
void renderer_buffer_init(Renderer* self, RendererBuffer buffer);
void renderer_buffer_free(Renderer* self, RendererBuffer buffer);
void renderer_free(Renderer* self);
void renderer_update(Renderer* self);
void renderer_present(Renderer* self);
void renderer_buffer_unbind(void);

void renderer_world_position_from_window_position
(
	Renderer* self, 
	vec2 windowPosition, 
	vec2 rendererPosition
);
