#pragma once

#include "component/COMPONENT_COMMON.h"

#include "../../engine/vector.h"

#define ECS_FUNCTION_COUNT ECS_FUNCTION_DRAW + 1
typedef enum ECSFunctionType
{
	ECS_FUNCTION_ADD,
	ECS_FUNCTION_DELETE,
	ECS_FUNCTION_TICK,
	ECS_FUNCTION_UPDATE,
	ECS_FUNCTION_DRAW
} ECSFunctionType;

typedef struct ECS ECS;
typedef struct Input Input;
typedef struct Renderer Renderer;
typedef struct Postprocessing Postprocessing;
typedef struct Resources Resources;
typedef u32 EntityID;

typedef void (*ECSFunction)(void*, ECS*);

typedef struct ECSComponent
{
	EntityID id;
	bool isDisabled;
	bool isFunctionDisabled[ECS_FUNCTION_COUNT];
} ECSComponent;

typedef struct ECSSystem
{
	ECSFunction functions[ECS_FUNCTION_COUNT];
} ECSSystem;

typedef struct ECSComponentInfo
{
	ECSSystem system;
	ECSComponentType type;
	size_t size;
} ECSComponentInfo;

typedef struct ECSComponentList
{
	ECS* ecs;
	ECSSystem system;
	ECSComponentType type;
	Vector components; 
} ECSComponentList;

typedef struct ECS
{
	Input* input;
	Renderer* renderer;
	Resources* resources;
	Postprocessing* postprocessing;
	ECSComponentList lists[ECS_COMPONENT_COUNT];
	EntityID nextID;
} ECS;

