#include "component_game_object.h"

void
component_game_object_init
(
	ComponentGameObject* self, 
	ECS* ecs,
	Texture* texture, 
	RendererBuffer buffer,
	vec2 size, 
	vec3 position
)
{
	ComponentTextureQuad* textureQuad;
	ComponentPhysics* physics;

	textureQuad = ecs_component_get(ecs, ECS_COMPONENT_TEXTURE_QUAD, self->component.id);
	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->component.id);

	component_texture_quad_init
	(
		textureQuad, 
		ecs, 
		texture, 
		&ecs->resources->shaders[SHADER_TEXTURE_QUAD],
		COMPONENT_TEXTURE_QUAD_FLIP_DEFAULT,
		ORIGIN_CENTER,
		buffer,
		COMPONENT_TEXTURE_QUAD_ROTATION_DEFAULT,
		size, 
		(f32*)COMPONENT_TEXTURE_QUAD_SCALE_DEFAULT,
		(f32*)COMPONENT_TEXTURE_QUAD_UV_MIN_DEFAULT,
		(f32*)COMPONENT_TEXTURE_QUAD_UV_MAX_DEFAULT,
		position,
		(f32*)COMPONENT_TEXTURE_QUAD_OFFSET_DEFAULT,
		(f32*)COMPONENT_TEXTURE_QUAD_COLOR_DEFAULT
	);

	glm_vec3_copy(position, physics->position);
}

void
component_game_object_add(ComponentGameObject* self, ECS* ecs)
{
	ecs_components_add(ecs, COMPONENT_GAME_OBJECT_DEPENDENCIES, COMPONENT_GAME_OBJECT_DEPENDENCY_COUNT, self->component.id);
}

void
component_game_object_tick(ComponentGameObject* self, ECS* ecs)
{
	ComponentTextureQuad* textureQuad;
	ComponentPhysics* physics;

	textureQuad = ecs_component_get(ecs, ECS_COMPONENT_TEXTURE_QUAD, self->component.id);
	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->component.id);

	glm_vec3_copy(physics->position, textureQuad->position);
}
