#include "component_outline.h"

static void _component_outline_texture_quad_set(ComponentOutline* self, ECS* ecs);

static void
_component_outline_texture_quad_set(ComponentOutline* self, ECS* ecs)
{
	ComponentTextureQuad* textureQuadOutline;
	ComponentTextureQuad* textureQuad;

	textureQuad = ecs_component_get(ecs, ECS_COMPONENT_TEXTURE_QUAD, self->component.id);
	
	if (!self->isSpriteInit)
	{
		self->spriteID = entity_sprite_add
		(
			ecs, 
			textureQuad->texture, 
			textureQuad->buffer,
			textureQuad->size,
			textureQuad->position
		);

		self->isSpriteInit = true;
	}

	textureQuadOutline = ecs_component_get(ecs, ECS_COMPONENT_TEXTURE_QUAD, self->spriteID);

	memcpy(textureQuadOutline, textureQuad, sizeof(ComponentTextureQuad));

	textureQuadOutline->component.id = self->spriteID;
	textureQuadOutline->position[2] += COMPONENT_OUTLINE_Z_OFFSET;
	glm_vec4_copy(self->color, textureQuadOutline->color);
	glm_vec2_adds(textureQuadOutline->size, self->size, textureQuadOutline->size);
	textureQuadOutline->shader = &ecs->resources->shaders[SHADER_COLORED_TEXTURE_QUAD];
}

void
component_outline_init(ComponentOutline* self, ECS* ecs, vec4 color, f32 size)
{
	glm_vec4_copy(color, self->color);
	self->size = size;
}

void
component_outline_add(ComponentOutline* self, ECS* ecs)
{
	component_outline_init(self, ecs, (f32*)COMPONENT_OUTLINE_COLOR, COMPONENT_OUTLINE_SIZE);
}

void
component_outline_delete(ComponentOutline* self, ECS* ecs)
{
	ecs_entity_delete(ecs, self->spriteID);
}

void
component_outline_tick(ComponentOutline* self, ECS* ecs)
{
	_component_outline_texture_quad_set(self, ecs);
}
