#pragma once

#include "../../ecs_entity.h"

#include "../../../../engine/rectangle.h"
#include "../../../../engine/font.h"
#include "../../../render/texture_quad.h"
#include "../../../resource/resource_shader.h"

#define COMPONENT_TEXT_LIMIT 1024
#define COMPONENT_TEXT_STRING_MAX 1024

#define COMPONENT_TEXT_COLOR_DEFAULT (f32*)OPAQUE

static const vec2 COMPONENT_TEXT_SCALE_DEFAULT = {1.0f, 1.0f};
static const vec3 COMPONENT_TEXT_OFFSET_DEFAULT = {0.0f, 0.0f, 0.0f};

typedef struct ComponentText
{
	ECSComponent component;
	Shader* shader;
	RendererBuffer buffer;
	vec3 position;
	vec4 color;
	vec2 scale;
	vec3 offset;
	Font* font;
	char string[COMPONENT_TEXT_STRING_MAX];
	u32 length;
	s32 wrap;
} ComponentText;

void component_text_glyph_size_get(ComponentText* self, Texture* texture, vec2 size);
void component_text_glyph_draw(ComponentText* self, ECS* ecs, Texture* texture, vec3 position);
void component_text_rectangle_get(ComponentText* self, Rectangle* rectangle);
void component_text_string_set(ComponentText* self, const char* string, u32 length);
void component_text_add(ComponentText* self, ECS* ecs);
void component_text_draw(ComponentText* self, ECS* ecs);

void
component_text_init
(
	ComponentText* self, 
	ECS* ecs,
	Font* font,
	Shader* shader,
	RendererBuffer buffer,
	const char* string,
	const vec2 scale,
	const vec3 position,
	const vec3 offset,
	const vec4 color,
	u32 length,
	s32 wrap
);

static const ECSComponentInfo COMPONENT_TEXT_INFO = 
{
	.system = 
	{
		.functions = 
		{ 
			(ECSFunction)component_text_add, 
			NULL,
			NULL,
			NULL,
			(ECSFunction)component_text_draw
		}
	},
	.type = ECS_COMPONENT_TEXT,
	.size = sizeof(ComponentText)
};
