#include "ecs_component.h"

void*
ecs_component_get(ECS* self, ECSComponentType type, EntityID id)
{
	return ecs_component_list_get(&self->lists[type], id);
}

void*
ecs_component_from_index_get(ECS* self, ECSComponentType type, u32 index)
{
	return vector_get(&self->lists[type].components, index);
}

void*
ecs_component_add(ECS* self, ECSComponentType type, EntityID id)
{
	void* component;
	ECSFunction add;
	
	component = ecs_component_get(self, type, id);

	if (component)
		return component;

	component = vector_push(&self->lists[type].components, NULL);

	memcpy(component, (void*)&id, sizeof(EntityID));

	add = self->lists[type].system.functions[ECS_FUNCTION_ADD];

	if (add)
		add(component, self);

	return component;
}

void
ecs_component_delete(ECS* self, ECSComponentType type, EntityID id)
{
	for (s32 i = 0; i < (s32)self->lists[type].components.count; i++)
	{
		EntityID checkID;
		void* component;

		component = vector_get(&self->lists[type].components, i);

		memcpy(&checkID, component, sizeof(EntityID));

		if (checkID == id)
		{
			ECSFunction delete;

			delete = self->lists[type].system.functions[ECS_FUNCTION_DELETE];

			if (delete)
				delete(component, self);

			vector_remove(&self->lists[type].components, i);
		}
	}
}

void
ecs_components_add(ECS* self, const ECSComponentType* types, u32 count, EntityID id)
{
	for (s32 i = 0; i < (s32)count; i++)
		ecs_component_add(self, types[i], id);
}

void
ecs_components_delete(ECS* self, const ECSComponentType* types, u32 count, EntityID id)
{
	for (s32 i = 0; i < (s32)count; i++)
		ecs_component_delete(self, types[i], id);
}
