#pragma once

#include "../../engine/file.h"
#include "../../engine/shader.h"
#include "../../engine/font.h"
#include "../../engine/texture.h"
#include "../../engine/sound.h"
#include "../../engine/music.h"

#define SHADER_COUNT SHADER_POSTPROCESS_TEXTURE_QUAD + 1
typedef enum ShaderType
{
	SHADER_TEXTURE_QUAD,
	SHADER_COLORED_TEXTURE_QUAD,
	SHADER_POSTPROCESS_TEXTURE_QUAD
} ShaderType;

typedef struct ShaderPaths
{
	const char* vertex;
	const char* fragment;
} ShaderPaths;

static const ShaderPaths SHADER_PATHS[SHADER_COUNT] =
{
	{
		.vertex = "res/shader/texture_quad.vs",
		.fragment = "res/shader/texture_quad.fs"
	},
	{
		.vertex = "res/shader/texture_quad.vs",
		.fragment = "res/shader/colored_texture_quad.fs"
	},

	{
		.vertex = "res/shader/texture_quad.vs",
		.fragment = "res/shader/postprocess_texture_quad.fs"
	},
};

#define TEXTURE_COUNT TEXTURE_CURSOR + 1
typedef enum TextureType
{
	TEXTURE_TEST,
	TEXTURE_TEST2,
	TEXTURE_CURSOR,
} TextureType;

static const char* TEXTURE_PATHS[TEXTURE_COUNT] =
{
	"res/gfx/test.png",
	"res/gfx/test2.png",
	"res/gfx/cursor.png"
};

#define FONT_COUNT FONT_TEST + 1
typedef enum FontType
{
	FONT_TEST
} FontType;

static const char* FONT_PATHS[FONT_COUNT] =
{
	"res/font/test.ttf"
};

static const u32 FONT_SIZES[FONT_COUNT] =
{
	32
};

#define SOUND_COUNT SOUND_TEST + 1
typedef enum SoundType
{
	SOUND_TEST
} SoundType;

static const char* SOUND_PATHS[SOUND_COUNT] =
{
	"res/sound/test.ogg"
};

#define MUSIC_COUNT MUSIC_TEST + 1
typedef enum MusicType
{
	MUSIC_TEST
} MusicType;

static const char* MUSIC_PATHS[MUSIC_COUNT] =
{
	"res/music/test.ogg"
};

typedef struct Resources
{
	Shader shaders[SHADER_COUNT];
	Texture textures[TEXTURE_COUNT];
	Music music[MUSIC_COUNT];
	Sound sounds[SOUND_COUNT]; 
	Font fonts[FONT_COUNT];
} Resources;
