#include "state.h"

static void _state_set(State* self, StateType type);

static void
_state_set(State* self, StateType type)
{
	self->type = type;

	resource_shader_state_init
	(
		self->ecs->resources, 
		STATE_SHADERS[self->type], 
		STATE_SHADER_COUNT[self->type]
	);

	resource_texture_state_init
	(
		self->ecs->resources, 
		STATE_TEXTURES[self->type], 
		STATE_TEXTURE_COUNT[self->type]
	);

	resource_font_state_init
	(
		self->ecs->resources, 
		STATE_FONTS[self->type], 
		STATE_FONT_COUNT[self->type]
	);

	resource_sound_state_init
	(
		self->ecs->resources, 
		STATE_SOUNDS[self->type], 
		STATE_SOUND_COUNT[self->type]
	);

	resource_music_state_init
	(
		self->ecs->resources, 
		STATE_MUSIC[self->type], 
		STATE_MUSIC_COUNT[self->type]
	);

	switch (self->type)
	{
		case STATE_GAMEPLAY:
			gameplay_init(&self->states.gameplay, self->ecs);
			break;
		case STATE_TITLE:
			title_init(&self->states.title, self->ecs);
			break;
		default:
			break;
	}
}

void
state_init(State* self, ECS* ecs, StateType type)
{
	memset(self, '\0', sizeof(State));

	self->ecs = ecs;

	_state_set(self, type);
}

void
state_free(State* self)
{
	ecs_clear(self->ecs);

	resource_shader_state_free
	(
		self->ecs->resources, 
		STATE_SHADERS[self->type], 
		STATE_SHADER_COUNT[self->type]
	);

	resource_texture_state_free
	(
		self->ecs->resources, 
		STATE_TEXTURES[self->type], 
		STATE_TEXTURE_COUNT[self->type]
	);

	resource_font_state_free
	(
		self->ecs->resources, 
		STATE_FONTS[self->type], 
		STATE_FONT_COUNT[self->type]
	);

	resource_sound_state_free
	(
		self->ecs->resources, 
		STATE_SOUNDS[self->type], 
		STATE_SOUND_COUNT[self->type]
	);

	resource_music_state_free
	(
		self->ecs->resources, 
		STATE_MUSIC[self->type], 
		STATE_MUSIC_COUNT[self->type]
	);
}

void
state_change(State* self, StateType type)
{
	self->setType = type;

	self->isChanging = true;
}

void
state_tick(State* self)
{
	if (self->isChanging)
	{
		/*
		ComponentAnimationColorChange* animationColorChange;
		animationColorChange = ecs_component_get(&self->game->ecs, ECS_COMPONENT_ANIMATION_COLOR_CHANGE, self->fade);

		if (animationColorChange->isDone)
		{
		*/
			self->isChanging = false;
			state_free(self);

			memset(&self->states, '\0', sizeof(self->states));

			_state_set(self, self->setType);
		//}
	}
	else
	{
		switch (self->type)
		{
			case STATE_GAMEPLAY:
				gameplay_tick(&self->states.gameplay);
				break;
			case STATE_TITLE:
				title_tick(&self->states.title);
				break;
			default:
				break;
		}
	}
}
